class OpenIDConfig
  def self.fetch(tid: 'common')
    config_uri = URI("https://login.microsoftonline.com/#{tid}/v2.0/.well-known/openid-configuration")
    new(config: JSON.parse(Net::HTTP.get(config_uri)))
  rescue JSON::ParserError
    fail StandardError, 'Unable to fetch OpenId configuration for ' \
      "AzureAD tenant '#{tid}'."
  end

  def initialize(config:)
    @config = config
  end

  def issuer
    config['issuer']
  end

  def authorization_endpoint
    config['authorization_endpoint']
  end

  def jwks_uri
    config['jwks_uri']
  end

  def keys
    @signing_keys ||= KeySet.new(keys: fetch_signing_keys)
  end

  private

  attr_reader :config

  def fetch_signing_keys
    response = JSON.parse(Net::HTTP.get(URI(signing_keys_url)))
    response['keys']
  rescue JSON::ParserError
    raise StandardError, 'Unable to fetch AzureAD signing keys.'
  end

  def signing_keys_url
    return jwks_uri if jwks_uri
    fail StandardError, 'No jwks_uri in OpenId config response.'
  end
end
